package com.galaxis.manatee.capsule.util;

import com.galaxis.manatee.capsule.exception.DaoMissingException;
import com.galaxis.manatee.capsule.exception.SpecificationMissingException;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.Pageable;
import org.springframework.data.jpa.domain.Specification;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestParam;

import java.util.Map;

/**
 * @author zcj
 * @version 0.1
 */
public class BasicController<T,K> extends JpaConfigurator<T,K> {

    /**
     * 注入dao对象
     * @param galaxisRepository  dao对象
     */
    public void setDao(GalaxisRepository<T,K> galaxisRepository){
        this.dao=galaxisRepository;
    }

    /**
     * 保存对象实例
     * @param t 待保存对象
     * @return  保存请求结果
     * @throws  DaoMissingException dao对象缺失的异常
     */
    @PostMapping("/save")
    public ResponseEntity<T> save(@RequestBody T t) throws DaoMissingException {
        if(this.dao==null){
            throw new DaoMissingException();
        }
        if (t==null){
            throw new IllegalArgumentException();
        }
        T result=dao.save(t);
        try {
            return new ResponseEntity<>(result, HttpStatus.OK);
        }catch (IllegalArgumentException illegalArgumentException){
            illegalArgumentException.printStackTrace();
            return new ResponseEntity<>(HttpStatus.SERVICE_UNAVAILABLE);
        }
    }

    /**
     * 分页查询
     *
     * @param parameters 查询条件
     *                   key为xxxx_yy,xxxx为查询参数名，yy为查询条件，如eq，lk等.
     *                   例如：   status_eq,CREATE 表示查询status字段equal查询，值为CREATE
     *                   property_lik,%234% 表示查询property字段like查询，值为%234%
     *                   value为yy,yy,yy为参数的值，多个值中间用","分隔
     * @return 分页数据
     */
    @GetMapping("/findAllByParameters")
    public Page<T> findAllByParams(@RequestParam Map<String, String> parameters) throws DaoMissingException, SpecificationMissingException {
        if(this.dao==null){
            throw new DaoMissingException();
        }
        Specification<T> specification = this.getSpecification(parameters);
        Pageable pageable=this.getPageable(parameters);
        if (specification==null||pageable==null){
            throw new SpecificationMissingException();
        }else{
            return dao.findAll(specification, pageable);
        }
    }
}
