package com.galaxis.manatee.task;

import com.fasterxml.jackson.core.type.TypeReference;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.galaxis.manatee.constant.ChuanYunConstant;
import com.galaxis.manatee.dao.*;
import com.galaxis.manatee.entity.chuanyun.data.object.*;
import com.galaxis.manatee.entity.chuanyun.dto.Filter;
import com.galaxis.manatee.event.ImplCostEvent;
import com.galaxis.manatee.manager.ChuanYunManager;
import com.galaxis.manatee.service.BuildProjectService;
import com.galaxis.manatee.service.ProjectOrganizationStructureService;
import com.galaxis.manatee.service.WorkGroupService;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.ApplicationContext;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.stereotype.Service;
import org.springframework.util.StringUtils;

import java.math.BigDecimal;
import java.util.*;

/**
 * @author zcj
 * @version 0.1
 * @date 2020/5/1 4:25 下午
 */
@Slf4j
@Service
public class ChuanyunFinancialScheduledTask {

    @Autowired
    private ApplicationContext applicationContext;
    @Autowired
    private WorkGroupService workGroupService;
    @Autowired
    private BuildProjectService buildProjectService;
    @Autowired
    private ProjectOrganizationStructureService projectOrganizationStructureService;

    private final Integer pageSize = 100;
    private final ChuanYunManager chuanYunManager;
    private final ChuanyunReimbursementDao chuanyunReimbursementDao;
    private final ChuanyunCompanyDao chuanyunCompanyDao;
    private final ChuanyunSupplierDao chuanyunSupplierDao;
    private final ChuanyunGroupProjectDao chuanyunGroupProjectDao;
    private final ChuanyunUserCompanyDao chuanyunUserCompanyDao;
    private final ChuanyunUserDao chuanyunUserDao;
    private final ChuanyunReimbursementSubjectDao chuanyunReimbursementSubjectDao;

    public ChuanyunFinancialScheduledTask(ChuanYunManager chuanYunManager, ChuanyunReimbursementDao chuanyunReimbursementDao, ChuanyunCompanyDao chuanyunCompanyDao, ChuanyunSupplierDao chuanyunSupplierDao, ChuanyunGroupProjectDao chuanyunGroupProjectDao, ChuanyunUserCompanyDao chuanyunUserCompanyDao, ChuanyunUserDao chuanyunUserDao, ChuanyunReimbursementSubjectDao chuanyunReimbursementSubjectDao) {
        this.chuanYunManager = chuanYunManager;
        this.chuanyunReimbursementDao = chuanyunReimbursementDao;
        this.chuanyunCompanyDao = chuanyunCompanyDao;
        this.chuanyunSupplierDao = chuanyunSupplierDao;
        this.chuanyunGroupProjectDao = chuanyunGroupProjectDao;
        this.chuanyunUserCompanyDao = chuanyunUserCompanyDao;
        this.chuanyunUserDao = chuanyunUserDao;
        this.chuanyunReimbursementSubjectDao = chuanyunReimbursementSubjectDao;
    }

    /**
     * 将报销信息从氚云中同步
     */
    @Scheduled(cron = "0 0 2 * * *")
    private void reimbursementInformationSync() {
        getReimbursementFromChuanyun();
    }

    public void syncReimbursement() {
        getReimbursementFromChuanyun();
    }

    /**
     * 获取报销单
     */
    protected void getReimbursementFromChuanyun() {
        var objectMapper = new ObjectMapper();
        var start = 0;
        var totalCount = 0L;
        var flag = true;
        List<String> matchersString = new ArrayList<>();
//        matchersString.add("Status_2,1");
        matchersString.add("Status_6,[1;2;3]");
        while (flag) {
            try {
                //从氚云查询数据
                var filter = Filter.instance(start, start + pageSize, true, Filter.AND, matchersString);
                var chuanyunFindAllResponse = chuanYunManager.findAll(ChuanyunReimbursementDO.SCHEMA_CODE, filter);
                if (chuanyunFindAllResponse.getReturnData() == null) {
                    flag = false;
                    continue;
                }
                if (chuanyunFindAllResponse.getReturnData().getTotalCount() >= start + pageSize) {
                    start += pageSize;
                } else {
                    flag = false;
                }
                //转化为POJO
                List<ChuanyunReimbursementDO> result = objectMapper.convertValue(chuanyunFindAllResponse.getReturnData().getBizObjectArray(), new TypeReference<>() {
                });
                //获取项目总数
                totalCount = chuanyunFindAllResponse.getReturnData().getTotalCount();
                //保存
                result.parallelStream().forEach(chuanyunReimbursementDO -> {
                    try {
                        //找到以前的数据的话就删除
                        chuanyunReimbursementDao.delete(chuanyunReimbursementDao.findById(chuanyunReimbursementDO.getObjectId()).orElseThrow());
                    } catch (NoSuchElementException e) {
                        //没有之前的数据不做处理
                    }

                    //将报销单中关联信息的id转化为具体的数据
                    try {
                        ChuanyunCompanyDO chuanyunCompanyDO = chuanyunCompanyDao.findById(chuanyunReimbursementDO.getBeneficiaryId()).orElseThrow();
                        if ("对公".equals(chuanyunReimbursementDO.getPrivateOrPublic()) && !StringUtils.isEmpty(chuanyunReimbursementDO.getSupplierId())) {
                            ChuanyunSupplierDO chuanyunSupplierDO = chuanyunSupplierDao.findById(chuanyunReimbursementDO.getSupplierId()).orElseThrow();
                            chuanyunReimbursementDO.setSupplierCode(chuanyunSupplierDO.getSupplierCode());
                        }
                        StringBuilder chuanyunGroupProjectCode = new StringBuilder();
                        if (!StringUtils.isEmpty(chuanyunReimbursementDO.getGroupProjectId())) {
                            ChuanyunGroupProjectDO chuanyunGroupProjectDO = chuanyunGroupProjectDao.findById(chuanyunReimbursementDO.getGroupProjectId()).orElseThrow();
                            chuanyunReimbursementDO.setGroupProjectCode(chuanyunGroupProjectDO.getProjectCode());
                            chuanyunGroupProjectCode.append(chuanyunGroupProjectDO.getProjectCode());
                        }
                        chuanyunReimbursementDO.setReimburseUserId(chuanyunReimbursementDO.getReimburseUserObject().getObjectId());
                        ChuanyunUserDO chuanyunUserDO = chuanyunUserDao.findById(chuanyunReimbursementDO.getReimburseUserId()).orElseThrow();
                        List<ChuanyunUserCompanyDO> userCompanyList = chuanyunUserCompanyDao.findAllByUserId(chuanyunReimbursementDO.getReimburseUserId());
                        Optional<ChuanyunUserCompanyDO> first = userCompanyList.stream().findFirst();
                        if (first.isPresent()) {
                            ChuanyunUserCompanyDO chuanyunUserCompanyDO = first.get();
                            chuanyunReimbursementDO.setReimburseUserCostCode(chuanyunUserCompanyDO.getCostCode());
                        }
//                        if (userCompanyList.size() != 1) {
//                            log.error(chuanyunUserDO.getName() + chuanyunReimbursementDO.getReimburseUserId());
//                        } else {
//                            ChuanyunUserCompanyDO chuanyunUserCompany = userCompanyList.get(0);
//                            chuanyunReimbursementDO.setReimburseUserCostCode(chuanyunUserCompany.getCostCode());
//                        }
//                        ChuanyunCostDO chuanyunCostDO=chuanyunCostDao.findById(chuanyunReimbursementDO.getBeneficiaryCostId()).orElseThrow();
                        chuanyunReimbursementDO.setBusinessDate(chuanyunReimbursementDO.getCreatedTime());
                        chuanyunReimbursementDO.setReimburseUserCompanyCode(chuanyunCompanyDO.getErpCode());
                        chuanyunReimbursementDO.setReimburseUserCode(chuanyunUserDO.getEmployeeNumber());
                        chuanyunReimbursementDO.setBeneficiaryBankCode(chuanyunCompanyDO.getBankCode());
                        chuanyunReimbursementDO.setBeneficiaryBankAccount(chuanyunCompanyDO.getBankAccount());
//                        chuanyunReimbursementDO.setBeneficiaryCostCode(chuanyunCostDO.getCostCode());
                        chuanyunReimbursementDO.setCurrency("C001");
                        chuanyunReimbursementDO.setExchangeRate(new BigDecimal("1"));
                        chuanyunReimbursementDO.setSettlementMethod("101");
                        chuanyunReimbursementDO.setErpFlag(0);
                        chuanyunReimbursementDO.getDetailList().forEach(chuanyunReimbursementDetailDO -> {
                            ChuanyunReimbursementSubjectDO chuanyunReimbursementSubjectDO = chuanyunReimbursementSubjectDao.findById(chuanyunReimbursementDetailDO.getReimburseSubjectId()).orElseThrow();
                            chuanyunReimbursementDetailDO.setReimburseSubjectCode(chuanyunReimbursementSubjectDO.getSubjectCode());
                            chuanyunReimbursementDetailDO.setReimburseSubjectName(chuanyunReimbursementSubjectDO.getSubjectName());
                            chuanyunReimbursementDetailDO.setReimburseUserCompanyCode(chuanyunCompanyDO.getErpCode());
                            chuanyunReimbursementDetailDO.setGroupProjectCode(chuanyunGroupProjectCode.toString());
//                            chuanyunReimbursementDetailDO.setReimburseUserCostCode(chuanyunCostDO.getCostCode());
                        });
                        if (ChuanYunConstant.NEED_WORK_GROUP_PROJECT_TYPE.contains(chuanyunReimbursementDO.getPrintingProjectType())) {
                            // 设置工作组名称
                            String workGroupId = chuanyunReimbursementDO.getWorkGroupId();
                            if (!Objects.isNull(workGroupId) && StringUtils.hasLength(workGroupId)) {
                                ChuanyunWorkGroupDO workGroupDO = workGroupService.findByObjectId(workGroupId);
                                if (workGroupDO != null) {
                                    chuanyunReimbursementDO.setWorkGroupName(workGroupDO.getWorkGroupName());
                                } else {
                                    // 查询历史数据中是否存在工作组
                                    ChuanyunHistoryWorkGroupDO inHistoryWorkGroup = workGroupService.findInHistoryByObjectId(workGroupId);
                                    chuanyunReimbursementDO.setWorkGroupName(inHistoryWorkGroup.getWorkGroupName());
                                }
                            } else {
                                // 没有填写工作组信息则任意选择该成员所在工作组
                                // 通过项目号查询组织架构中的工作组
                                List<ChuanyunBuildProject> allBuildProjectList = buildProjectService.findAllByProjectCode(chuanyunReimbursementDO.getGroupProjectCode());
                                if (!Objects.isNull(allBuildProjectList)) {
                                    allBuildProjectList.forEach(chuanyunBuildProject -> {
                                        List<ChuanyunProjectOrganizationStructureDO> posList = projectOrganizationStructureService.findAllByProjectId(chuanyunBuildProject.getObjectId());
                                        if (!posList.isEmpty()) {
                                            for (ChuanyunProjectOrganizationStructureDO chuanyunProjectOrganizationStructureDO : posList) {
                                                // 查询工作组
                                                List<ChuanyunWorkGroupDO> workGroupList = workGroupService.findByParentObjectId(chuanyunProjectOrganizationStructureDO.getObjectId());
                                                if (!Objects.isNull(workGroupList)) {
                                                    for (ChuanyunWorkGroupDO workGroupDO : workGroupList) {
                                                        List<String> workGroupMamberUserIdList = workGroupDO.getSubWorkGroupMemberObject().stream().map(ChuanyunWorkGroupMemberDO::getUserId).toList();
                                                        if (workGroupMamberUserIdList.contains(chuanyunReimbursementDO.getReimburseUserId())) {
                                                            chuanyunReimbursementDO.setWorkGroupName(workGroupDO.getWorkGroupName());
                                                            chuanyunReimbursementDO.setWorkGroupId(workGroupDO.getObjectId());
                                                            break;
                                                        }

                                                    }
                                                }
                                            }
                                        }
                                    });
                                }
                            }
                            // 任为空，则赋予默认值
                            if (!StringUtils.hasLength(chuanyunReimbursementDO.getWorkGroupId()) && !StringUtils.hasLength(chuanyunReimbursementDO.getWorkGroupName())) {
                                chuanyunReimbursementDO.setWorkGroupId(ChuanYunConstant.DEFAULT_WORK_GROUP);
                                chuanyunReimbursementDO.setWorkGroupName(ChuanYunConstant.DEFAULT_WORK_GROUP_NAME);
                            }
                        }
                    } catch (Exception e) {
                        log.warn(e.getMessage());
                        e.printStackTrace();
                    }

                });
                chuanyunReimbursementDao.saveAll(result);
                log.info("start: {}", start);
            } catch (Exception e) {
                log.error(e.getMessage());
                e.printStackTrace();
            }
        }
        log.info("报销信息保存" + totalCount);
        // 发布事件汇总工作组实施费用
        applicationContext.publishEvent(new ImplCostEvent(this));
    }


}
